/**
* Filename: SerializableVector.java (in org.openbandy.io)
* This is part of the OpenBandy project.

* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.5
* 
* (c) Copyright 2007, ALL RIGHTS RESERVED.
*
* This class implements a serializable vector. It can contain
* an unlimited number of serializable objects of the same type.
* 
* It is possible to name instances of this class using the
* setSerializationName() method.
*/

package org.openbandy.io;

import java.util.Enumeration;
import java.util.Vector;

import org.openbandy.io.storage.ObjectIdString;
import org.openbandy.io.storage.Storage;
import org.openbandy.io.storage.StorageSerializer;
import org.openbandy.io.storage.StorageValueContainer;
import org.openbandy.log.LogImpl;
import org.openbandy.service.LogService;


public class SerializableVector implements Serializable{
    
    /* The vectors id */
    private int vectorId = SerializationIdentifiers.NO_ID;
    
    /* The vectors serialization name.
     * This is needed in order to allow objects to have multiple
     * SerializableVectors, even when they contain the same type
     * of objects */
    private String serializationName = "";
    
    /* The events validity in minutes */
    protected java.util.Vector vector = new java.util.Vector();
    
    /* The class name of contained serializable objects */
    private String containedObjectsClassName = "";
    
    /* The number of serializable objects in vector */
    private int size = 0;
    
    /* Field names stored as constants, will be used by other classes */
    public final static String CONTAINED_OBJS_CLASSNAME = "containedObjectsClassName";
    public final static String CONTAINED_OBJS_IDS = "containedObjectIds";
    
    /* Reference to the storage that last manipulated the vector, might be null */
    //TODO check if this is really necessary
    private Storage storage;
    
    /* Constructor */
    public SerializableVector(){
    }
    
    /* ********** Getter and Setter ************ */
    
    public String getSerializationName() {
        return serializationName+getIdName();
    }

    public void setSerializationName(String serializationName) {
        this.serializationName = serializationName;
    }
    
    public void setContainedObjectsClassName(String containedObjectsClassName){
        this.containedObjectsClassName = containedObjectsClassName;
    }
    
    public String getContainedObjectsClassName(){
        return containedObjectsClassName;
    }
    
    public int size(){
        return size;
    }
    
    /**
     * returns true if the serializable could be added, false
     * if it could not be added because it has a different type
     * than already stored objects
     * 
     * @param serializable
     * @return
     */
    public boolean add(Serializable serializable){
        if(size == 0){
            containedObjectsClassName = serializable.getClass().getName();
            vector.addElement(serializable);
            size++;
            return true;
        }else if(serializable.getClass().getName().equals(containedObjectsClassName)){
            vector.addElement(serializable);
            size++;
            return true;
        }else{
            LogService.warn(this, "Can not add objects of type "+serializable.getClass().getName());
            return false;
        }
    }
    
    public Serializable firstElement(){
        return ((Serializable)vector.firstElement());
    }
    
    /**
     * If the serializable object is contained in this vector, it will
     * be deleted. If it has already been stored to a storage, it will
     * also be deleted from the storage.
     * 
     * @param serializable
     * @return true if removed successfully
     * 
     */
    public boolean remove(Serializable serializable){
    	
    	/* check if this object is contained in this vector */
    	if(contains(serializable)){
    		
    		/* check if this serializable object is stored in */
    		if(storage != null){
    			
    			/* check if this object is in the storage and deleted if so */
    			if(storage.existsInStorage(serializable) > 0){
    				if(storage.delete(serializable)){

    					/* delete the object from the actual vector */
        				vector.removeElement(serializable);
        				size--;
                        if(size == 0){
                        	this.containedObjectsClassName = "";
                        }
                        return true;
    				}
    				else {
    					LogService.warn(this, "Could not delete: "+serializable.getLegibleString());
    					return false;
    				}
    			}
    			else {
    				
    				/* just delete the object from the actual vector */
    				vector.removeElement(serializable);
    				size--;
                    if(size == 0){
                    	this.containedObjectsClassName = "";
                    }
                    return true;
    			}
    		}
    		else {

				/* just delete the object from the actual vector */
				vector.removeElement(serializable);
				size--;
                if(size == 0){
                	this.containedObjectsClassName = "";
                }
                return true;
        	}
    	}
    	else {
    		LogService.warn(this, "Serializable not contained in this vector: "+serializable.getLegibleString());
    		return false;
    	}
    }
    
    /**
     * @return true if removed successfully
     */
    public boolean removeFirstElement(){
        if(size > 0){
        	Serializable serializable = (Serializable)vector.firstElement();
        	return remove(serializable);
        }
        return false;
    }
    
    public Vector getVector(){
        return vector;
    }
    
    public Enumeration elements(){
        return vector.elements();
    }
    
    /**
     * Checks if an object with the same id is in
     * the vector.
     * 
     * @param serializable The object to look for.
     * @return True if an object of the same type with the same id is in the vector. False else.
     */
    public boolean contains(Serializable serializable){
    	if(! serializable.getClass().getName().equals(containedObjectsClassName)){
    		return false;
    	}
    	
    	Enumeration elements = vector.elements();
    	while(elements.hasMoreElements()){
    		Serializable containedSerializable = (Serializable)elements.nextElement();
    		if(containedSerializable.getIdValue() == serializable.getIdValue()){
    			return true;
    		}
    	}
    	return false;
    }
    
    /* *************** Methods for Serialization ***************** */

    public void serialize(Serializer serializer, SerializationBuffer buffer) {
    	try{
        	/* set the storage reference */
    		storage = ((StorageSerializer)serializer).getStorage();
    	}catch(ClassCastException cce){
    		/* do nothing */
    	}
    	
    	/* serialize attributes */
        buffer.container().addInt("size", size);
        buffer.container().addString(CONTAINED_OBJS_CLASSNAME, containedObjectsClassName);
        
        /* serialize contained objects */
        Enumeration elements = elements();
        while(elements.hasMoreElements()){
            try{
                Serializable s = (Serializable)elements.nextElement();
                serializer.serializeObject(s, buffer);
            }catch(ClassCastException cce){
                LogService.warn(this, "Object in vector is not Serializable");
            }
        }
    }

    public void deserialize(Serializer serializer, SerializationBuffer buffer) {
        int vectorSize = buffer.container().getInt("size");
        if(vectorSize>0){
            if(containedObjectsClassName != ""){
                for(int i=0; i<vectorSize; i++){
                    /* create new contained object */
                    Serializable containedObject = SerializableFactory.getNewSerializableObject(containedObjectsClassName);
                    serializer.deserializeObject(containedObject, buffer);
                    add(containedObject);
                }
            }else{
                LogService.warn(this, "Class name of contained objects is not set");
            }
        }else{
            containedObjectsClassName = "";
        }
    }
    
    public void deserializeSuper(Serializer serializer, SerializationBuffer buffer) {
        LogService.debug(this, this.getClass().getName()+" does not have a super type");
    }
    
    /**
     * If the vector is serialized to a storage, check if an older version
     * if the vector already exists in the persistent storage. If so, delete
     * the affected entries (i.e. objects that are in the stored vector but
     * not in the actual object) from the persistent storage.
     * 
     * @param container The vectors value buffer.container().
     */
    public void prepareStorageSerialization(Storage storage, StorageSerializer storageSerializer){
    	/* check if this object already exists in storage */
    	if(SerializationIdentifiers.isValidId(storage.existsInStorage(this))){
    		
    		/* load the version that is stored */
    		SerializableVector vectorInStorage = new SerializableVector();
    		storageSerializer.load(vectorInStorage, this.getIdValue());
    		
    		/* delete entries that are void in the actual version of the vector */
    		for(Enumeration elements = vectorInStorage.elements(); elements.hasMoreElements();){
    			Serializable serializableInStorage = (Serializable)elements.nextElement();
    			if(!this.contains(serializableInStorage)){
    				storage.delete(serializableInStorage);
    			}
    		}
    	}
    }
    
    /**
     * This method prepares the Vector for deserialization by a
     * StorageSerializer.
     * 
     * @param container The vectors value buffer.container().
     */
    public void prepareStorageDeserialization(StorageValueContainer container){
        /* set class name of contained objects */
        containedObjectsClassName = container.getString(CONTAINED_OBJS_CLASSNAME);
        
        /* set contained object ids in container */
        String containedObjectIds = container.getString(CONTAINED_OBJS_IDS);
        container.setObjectIdString(new ObjectIdString(containedObjectIds));
    }
    
    public String getLegibleString() {
        String legibleString = "Vector : "+vectorId;
        if(size>0){
            legibleString = legibleString+LogImpl.VALUE_DELIMITER+size+LogImpl.VALUE_DELIMITER;
            Enumeration elements = vector.elements();
            while(elements.hasMoreElements()){
                try{
                    Serializable serializable = (Serializable)elements.nextElement();
                    legibleString = legibleString+serializable.getLegibleString()+LogImpl.VALUE_DELIMITER;
                }catch(ClassCastException cce){
                    LogService.warn(this, "Object in vector is not Serializable");
                }
            } 
        }
        return legibleString;
    }

    public boolean isEqualShallow(Serializable serializableObject) throws ClassCastException {
        SerializableVector comparingVector = (SerializableVector)serializableObject;
        boolean equal = (this.size == comparingVector.size);
        if(size > 0){
            equal = equal && this.containedObjectsClassName.equals(comparingVector.containedObjectsClassName);
        }
        return equal;
    }
    
    public boolean isEqual(Serializable serializableObject) throws ClassCastException {
        boolean equal = isEqualShallow(serializableObject);
        if(equal){
            SerializableVector comparingVector = (SerializableVector)serializableObject;
            
            Enumeration elements = vector.elements();
            Enumeration comparingElements = comparingVector.elements();
            while(elements.hasMoreElements()){
                try{
                    Serializable serializable = (Serializable)elements.nextElement();
                    Serializable comparingSerializable = (Serializable)comparingElements.nextElement();
                    equal = equal && (serializable.isEqual(comparingSerializable));
                    if(!equal){
                        return equal;
                    }
                }catch(ClassCastException cce){
                    LogService.warn(this, "Object in vector is not Serializable");
                }
            }   
        }
        return equal;
    }
    
    public Serializable getNewInstance() {
    	return new SerializableVector();
    }
    
    public Serializable getCopy(){
        SerializableVector clone = new SerializableVector();
        Enumeration elements = vector.elements();
        while(elements.hasMoreElements()){
            try{
                Serializable serializable = (Serializable)elements.nextElement();
                clone.add(serializable.getCopy());
            }catch(ClassCastException cce){
                LogService.warn(this, "Object in vector is not Serializable");
            }
        } 
        return clone;
    }
    

    public String getIdName() {
        return "vectorId";
    }

    public int getIdValue() {
        return vectorId;
    }

    public void setIdValue(int idValue) {
        vectorId = idValue;
    }

    public void getAttributeNames(java.util.Vector attributes) {
        attributes.addElement("size");
        attributes.addElement(CONTAINED_OBJS_CLASSNAME);
        attributes.addElement(CONTAINED_OBJS_IDS);
    }

    public void getReferences(java.util.Vector references) {
        Enumeration elements = vector.elements();
        while(elements.hasMoreElements()){
            references.addElement(((Serializable)elements.nextElement()));
        }
    }

    public void getReferencedIdNames(java.util.Vector referenceNames) {
        /* do nothing, references are explicitly store in the field containedObjectIds */
    }

    public Serializable getSuperCopy() {
        return null;
    }
}
