/**
* Filename: Storage.java (in org.openbandy.io.storage)
* This is part of the OpenBandy project.

* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.0
* 
* (c) Copyright 2007, ALL RIGHTS RESERVED.
*
* Administers a list of every currently active StorageConnection.
* (@see ch.ethz.inf.vs.bandy.storage.StorageConnection)
*/

package org.openbandy.io.storage;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.openbandy.io.Serializable;
import org.openbandy.service.LogService;

//TODO comment all methods in this class!

public abstract class Storage {

	/* A list of all open connections, avoid duplicates */
	protected static Hashtable connections = new Hashtable();

	/**
	 * Return 0 if <code>databaseSerializable</code> does not already
	 * exist in database. If <code>databaseSerializable</code> exist,
	 * this method returns the id according to <code>databaseSerializable.getIdName()</code>.
	 * 
	 * @param databaseSerializable
	 * @return
	 */
	public abstract int existsInStorage(Serializable serializable);


	/**
	 * Deletes the serializable object, but not the reference objects.
	 * 
	 * @param serializable
	 * @return true if deletion was successful
	 */
	public abstract boolean deleteShallow(Serializable serializable);

	/**
	 * Deletes the serializable and all the objects it references.
	 * @param serializable
	 * @return true if deletion was successful
	 */
	public synchronized boolean delete(Serializable serializable) {
		if(serializable != null){
			/* delete the object */
			if(deleteShallow(serializable)){

				/* if successful, delete referenced objects */
				boolean success = true;
				Vector references = new Vector();
				serializable.getReferences(references);
				Enumeration elements = references.elements();
				while(elements.hasMoreElements()){
					Serializable referencedSerializable = (Serializable)elements.nextElement();
					success = success && delete(referencedSerializable);
				}
				return success;
			}
		}
		LogService.warn(this, "Object is null");
		return false;
	}

	public int size() {
		return connections.size();
	}

	/**
	 * Returns true if the connection was newly added
	 * 
	 * @param connection
	 * @return
	 */
	protected boolean addConnection(StorageConnection connection){
		if(!connections.containsKey(connection.getConnectionIdentifier())){
			connections.put(connection.getConnectionIdentifier(), connection);
			return true;
		}
		return false;
	}

	/**
	 * Returns true if the connection was removed
	 * 
	 * @param connection
	 * @return
	 */
	public boolean removeConnection(StorageConnection connection){
		if(connection != null){
			if(connection.isConnected()){
				connection.close();
			}
			return (connections.remove(connection.getConnectionIdentifier()) != null);
		}
		return false;
	}

	public void closeAllConnections() {
		Enumeration elements = connections.elements();
		while(elements.hasMoreElements()){
			StorageConnection conn = (StorageConnection) elements.nextElement();
			try{
				conn.close();
			}catch(Exception e){
				LogService.error(this, "Could not close storage connection.", e);
			}
		}
		connections.clear();

	}

	/**
	 *  Reset the storage with all it's meta structures and caches.
	 *  Use carefully!
	 */
	public abstract void reset();

	/**
	 *  Returns true if no object is contained within the storage.
	 */
	public abstract boolean isEmpty();

}
