/**
 * Filename: ConnectionHelper.java (in org.openbandy.net)
 * This file is part of the BandyClient project.
 *
 * www.BandyClient.org
 */

package org.openbandy.net;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import org.openbandy.io.Message;
import org.openbandy.io.stream.StreamSerializer;
import org.openbandy.service.LogService;
import org.openbandy.service.SerializerService;

/**
 * Helper class which provides static methods to read and write
 * message to and from a stream.
 *
 * Every transmission starts with an <code>int</code> which denotes
 * the content of the stream (e.g. ACK).
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.5
 * 
 */
public class ConnectionHelper {

	/* status */
	public static final int ACK = 0;
	public static final int VALID_MSG = 1;
	public static final int INVALID_MSG = 2;

	/**
	 * Tries to read a message from the input stream.
	 *
	 * @param dataInputStream Input stream from which to read
	 * @return A message or null if nothing could be read
	 */
	public static Message readMessage(DataInputStream dataInputStream){
		try {
			/* read status */
			int status = dataInputStream.readInt();
			if(status == VALID_MSG){

				/* get the number of bytes which must be read */
				int bufferLength = dataInputStream.readInt();

				/* create a new buffer */
				byte[] buffer = new byte[bufferLength];

				/* read into the buffer */
				dataInputStream.read(buffer);
				LogService.debug("ConnectionHelper", "Read "+(bufferLength)+" bytes");

				/* get message from buffer */
				ByteArrayInputStream bais = new ByteArrayInputStream(buffer);

				/* read message object from stream and return */
				return (Message) SerializerService.getObjectFromXmlStream(bais);
			}
			else if(status == INVALID_MSG) {
				LogService.warn("ConnectionHelper", "No response message from server, request is invalid message");
				return null;
			}
			else if(status == ACK) {
				LogService.debug("ConnectionHelper", "No response message from server");
				return null;
			}
		} catch (IOException ioe) {
			LogService.error("ConnectionHelper", "Could not read message: "+ioe.getMessage(), ioe);
		}
		return null;
	}

	/**
	 * Writes the message to the output stream given
	 *
	 * @param message Message to be written
	 * @param dataOutputStream Output stream on which to write the message
	 */
	public static void writeMessage(Message message, DataOutputStream dataOutputStream){
		try {
			/* serialize messsage into a stream using best compression  */
			ByteArrayOutputStream baos = SerializerService.getXmlStream(message, StreamSerializer.COMPRESSED_BEST);

			/* write status */
			dataOutputStream.writeInt(VALID_MSG);
			
			/* write size of message */
			dataOutputStream.writeInt(baos.size());

			/* write message */
			dataOutputStream.write(baos.toByteArray());
			dataOutputStream.flush();
			
			LogService.debug("ConnectionHelper", "Wrote message of size: "+baos.size());
		} catch (IOException ioe) {
			LogService.error("ConnectionHelper", "Could not write message: "+ioe.getMessage(), ioe);
		}
	}

	/**
	 * Send an status (ACK | VALID_MSG | INVALID_MSG) to the output stream given
	 * 
	 * @param status status to write (ACK | VALID_MSG | INVALID_MSG)
	 * @param dataOutputStream Output stream on which to write the status
	 */
	public static void writeStatus(int status, DataOutputStream dataOutputStream){
		try {
			dataOutputStream.writeInt(status);
			dataOutputStream.flush();
		} catch (IOException ioe) {
			LogService.error("ConnectionHelper", "Could not write status '"+status+"': "+ioe.getMessage(), ioe);
		}
	}
	
}
